extends CanvasLayer

var dialogue_file_path:String = "res://objects/characters/components/story_cutscene.json"
@onready var audio = $WriteAudio
var current_dialogue_id = 0
var dialogues = []
var is_dialogue_active = false

func _ready():
	$NinePatchRect.visible = true
	play()
	
func _process(delta):
	if is_dialogue_active and $NinePatchRect/Message.is_writing():
		if not audio.playing:
			audio.play()
	else:
		audio.stop()

func _input(event):
	if not is_dialogue_active:
		return
		
	if event.is_action_pressed("spacebar"):
		next_line()
	
func next_line():
	if $NinePatchRect/Message.is_writing():
		$NinePatchRect/Message.finish_text_immediately()
	else:
		current_dialogue_id += 1
		if current_dialogue_id >= len(dialogues):
			$Timer.start()
			$NinePatchRect.visible = false
			turn_on_the_player()
			return
		$NinePatchRect/Message.clear_text()
		$NinePatchRect/Name.text = dialogues[current_dialogue_id]['name']
		$NinePatchRect/Message.write(dialogues[current_dialogue_id]['text'], 50.0)

func play():
	if is_dialogue_active:
		return
	dialogues = load_dialogue()
	turn_off_the_player()
	is_dialogue_active = true
	$NinePatchRect.visible = true
	current_dialogue_id = -1
	next_line()
	
func load_dialogue():
	var file = FileAccess.open(dialogue_file_path, FileAccess.READ)
	var content = file.get_as_text()
	return JSON.parse_string(content)
	
func _on_timer_timeout():
	is_dialogue_active = false

func turn_on_the_player():
	var player = get_tree().get_root().find_child("Grovekeeper", true, false)
	if player:
		player.set_active(true)

func turn_off_the_player():
	var player = get_tree().get_root().find_child("Grovekeeper", true, false)
	if player:
		player.set_active(false)
