class_name BuildContainer
extends ScrollContainer


const STRUCTURE_SLOT_SCN := preload("res://objects/UI/inventory/build/structure_slot.tscn")

@export var build_data: BuildDataResource
@export var inventory_data: InventoryDataResource

@onready var _container = $Structures
var _slots: Array[StructureSlot] = []


func _ready() -> void:
	if not inventory_data:
		inventory_data = Singletons.inventory_data_resource
	if not build_data:
		build_data = Singletons.build_data_resource
	
	setup_slot()
	
	build_data.chosen_structure_changed.connect(
		func(index: int) -> void:
			print("struct_select %d" % index)
	)
	inventory_data.update_all.connect(
		func() -> void:
			update_slots()
	)
	inventory_data.hotbar_changed.connect(update_slots.unbind(1))
	inventory_data.slot_changed.connect(update_slots.unbind(1))


func update_slots() -> void:
	for slot in _slots:
		slot.refresh_item_stack_data()


func setup_slot() -> void:
	for index in build_data.structure_resources.size():
		var structure_res := build_data.structure_resources[index]
		var slot: StructureSlot = STRUCTURE_SLOT_SCN.instantiate()
		
		_container.add_child(slot)
		_slots.append(slot)
		slot.index = index
		slot.set_structure_res(structure_res)
		slot.set_inventory_data(inventory_data)
		slot.set_build_data(build_data)
		slot.finalize()
