class_name StructureSlot
extends TextureRect


const MATERIAL_SLOT_SCN := preload("res://objects/UI/inventory/build/material_slot.tscn")

# given by container
@export var inventory_data: InventoryDataResource
@export var build_data: BuildDataResource
@export var quantity_label: Label

var index: int = 0
var structure_res: StructureResource : set = set_structure_res

var count = 0 :
	set(value):
		count = value
		quantity_label.text = str(count)

@onready var icon: TextureRect = $Icon
@onready var info_panel: Panel = $InfoPanel
@onready var structure_name_label: Label = %StructureNameLabel
@onready var materials_container: HBoxContainer = %MaterialsContainer


func _gui_input(event: InputEvent) -> void:
	if event is InputEventMouseButton:
		if event.is_pressed() and event.button_index == MOUSE_BUTTON_LEFT:
			if count <= 0 or not build_data.current_selected_structure == index:
				build_data.current_selected_structure = index
			else:
				build_data.current_selected_structure = -1
			get_viewport().set_input_as_handled()


func _on_mouse_entered() -> void:
	info_panel.show()


func _on_mouse_exited() -> void:
	info_panel.hide()


# refresh the state of item in the slot, need to be called by and connected to a signal from container 
func refresh_item_stack_data() -> void:
	var item_req_dict = {}
	for item_quantity: ItemQuantity in structure_res.item_req:
		item_req_dict[item_quantity.item_id] = item_quantity.quantity
	var item_count_dict := inventory_data.count_items(item_req_dict.keys())
	var counts = -1
	for key in item_req_dict.keys():
		var times = item_count_dict[key] / item_req_dict[key]
		counts = times if counts == -1 else min(times,counts)
	
	count = counts if counts >= 0 else 1
	if quantity_label:
		quantity_label.text = str(count)
	if count <= 0 and build_data.current_selected_structure == index:
		build_data.current_selected_structure = -1
	update_current_modulate_color()


func update_current_modulate_color():
	if count <= 0:
		self_modulate = Color(0.5, 0.5, 0.5 , 1)
	elif build_data and build_data.current_selected_structure == index:
		self_modulate = Color("#F9812A", 1)
	else:
		self_modulate = Color(1, 1, 1, 1)


func set_structure_res(value: StructureResource) -> void:
	structure_res = value
	
	structure_name_label.text = structure_res.name
	icon.texture = structure_res.icon
	for c in materials_container.get_children():
		c.queue_free()
	for item_quantity in structure_res.item_req:
		var material_slot: MaterialSlot = MATERIAL_SLOT_SCN.instantiate()
		materials_container.add_child(material_slot)
		material_slot.initialize(item_quantity)


func set_inventory_data(inv_data: InventoryDataResource) -> void:
	self.inventory_data = inv_data


func set_build_data(value: BuildDataResource) -> void:
	build_data = value


func finalize() -> void:
	refresh_item_stack_data()
	build_data.chosen_structure_changed.connect(
		func(index: int) -> void:
			update_current_modulate_color()
	)
