extends BaseInventorySlot
class_name EquipmentSlot

@export var equipment_index := 0
@export var item_texture: TextureRect
@export var allowed_item_name: String
@export var placeholder_texture: Texture2D

func _ready() -> void:
	if !quantity_label:
		quantity_label = $CenterContainer/Panel/QuantityLabel
	if !item_texture:
		item_texture = $CenterContainer/ItemTexture

func set_inventory_data(inv_data: InventoryDataResource) -> void:
	inventory_data = inv_data
	refresh_item_stack_data()

func refresh_item_stack_data() -> void:
	var item := get_item_data()
	var item_stack_res := get_item_stack_data()

	quantity_label.text = str(item_stack_res.quantity) if show_item_quantity_label(item_stack_res) else ""
	item_texture.texture = item.icon if item else placeholder_texture
	
	if !item:
		item_texture.self_modulate = Color('#404040')
	else:
		item_texture.self_modulate = Color('#fff')

func _get_drag_data(position: Vector2) -> Variant:
	if inventory_data.get_equipment(equipment_index):
		var preview := TextureRect.new()
		preview.texture = inventory_data.get_equipment(equipment_index).item_res.icon
		GlobalControl.global_set_drag_preview(preview)
	
	return self

func _drop_data(position: Vector2, data: Variant) -> void:
	var source_inv_slot := data as BaseInventorySlot
	var source_item := source_inv_slot.get_item_stack_data()
	var dest_item := self.get_item_stack_data()
	self.set_item_stack_data(source_item)
	source_inv_slot.set_item_stack_data(dest_item)

func get_item_data() -> ItemResource:
	var item_stack_res := get_item_stack_data()
	return item_stack_res.item if item_stack_res else null

func get_item_stack_data() -> ItemStackResource:
	return inventory_data.get_equipment(equipment_index)

func set_item_stack_data(item: ItemStackResource) -> void:
	inventory_data.insert_equipment(item, equipment_index)

# check if can drop item to this slot, 
func can_drop_item(itemStack: ItemStackResource) -> bool:
	if !allowed_item_name:
		return true
	if itemStack == null || itemStack.item == null:
		return true
	return itemStack.item.name == allowed_item_name
