class_name HotbarSlot
extends BaseInventorySlot


@export var hotbar_index := 0
@export var item_texture: TextureRect

func _ready() -> void:
	if !quantity_label:
		quantity_label = $CenterContainer/Panel/QuantityLabel
	if !item_texture:
		item_texture = $CenterContainer/ItemTexture

func get_current_modulate_color():
	if inventory_data.current_hotbar_index == hotbar_index:
		return Color("#F9812A", 1)
	else:
		return Color(1, 1, 1, 1)

func set_inventory_data(inv_data: InventoryDataResource) -> void:
	inventory_data = inv_data
	
	refresh_item_stack_data()
	self_modulate = get_current_modulate_color()
	
	inventory_data.chosen_hotbar_changed.connect(
		func(index: int) -> void:
			self_modulate = get_current_modulate_color()
	)


func can_drop_item(item: ItemStackResource) -> bool:
	return true

func _gui_input(event: InputEvent) -> void:
	if event is InputEventMouseButton:
		if event.is_pressed():
			inventory_data.current_hotbar_index = hotbar_index


func _get_drag_data(position: Vector2) -> Variant:
	if inventory_data.get_hotbar(hotbar_index):
		var preview := TextureRect.new()
		preview.texture = inventory_data.get_hotbar(hotbar_index).item_res.icon
		GlobalControl.global_set_drag_preview(preview)
	
	return self


func _drop_data(position: Vector2, data: Variant) -> void:
	var source_inv_slot := data as BaseInventorySlot
	var source_item := source_inv_slot.get_item_stack_data()
	var dest_item := self.get_item_stack_data()
	self.set_item_stack_data(source_item)
	source_inv_slot.set_item_stack_data(dest_item)

# NOTE: deprecated function
func refresh() -> void:
	refresh_item_stack_data()

func refresh_item_stack_data() -> void:
	var item := get_item_data()
	var item_stack_res := get_item_stack_data()
	
	quantity_label.text = str(item_stack_res.quantity) if show_item_quantity_label(item_stack_res) else ""
	item_texture.texture = item.icon if item else null

func get_item_data() -> ItemResource:
	var item_stack_res := get_item_stack_data()
	return item_stack_res.item_res if item_stack_res else null

func get_item_stack_data() -> ItemStackResource:
	return inventory_data.get_hotbar(hotbar_index)

func set_item_stack_data(item: ItemStackResource) -> void:
	inventory_data.insert_hotbar(item, hotbar_index)
