class_name InventorySlot
extends BaseInventorySlot


signal item_stack_res_put(item_stack_res: ItemStackResource)

@export var item_texture: TextureRect
@export var slot_index: int = 0


func _ready() -> void:
	quantity_label = $QuantityLabel
	if !quantity_label:
		quantity_label = $QuantityLabel


func set_inventory_data(inv_data: InventoryDataResource) -> void:
	inventory_data = inv_data
	refresh_item_stack_data()


func can_drop_item(item: ItemStackResource) -> bool:
	return true


func _get_drag_data(position: Vector2) -> Variant:
	if inventory_data.get_item(slot_index):
		var preview := TextureRect.new()
		preview.texture = inventory_data.get_item(slot_index).item_res.icon
		GlobalControl.global_set_drag_preview(preview)
	
	return self


func _drop_data(position: Vector2, data: Variant) -> void:
	var source_inv_slot := data as BaseInventorySlot
	var source_item := source_inv_slot.get_item_stack_data()
	var dest_item := self.get_item_stack_data()
	self.set_item_stack_data(source_item)
	source_inv_slot.set_item_stack_data(dest_item)


# NOTE: deprecated function
func update_item_stack_data() -> void:
	refresh_item_stack_data()


func refresh_item_stack_data() -> void:
	var item := get_item_data()
	var item_stack_res := get_item_stack_data()

	quantity_label.text = str(item_stack_res.quantity) if show_item_quantity_label(item_stack_res) else ""
	item_texture.texture = (item.icon if item else null)


func get_item_data() -> ItemResource:
	var item_stack_res := get_item_stack_data()
	return item_stack_res.item_res if item_stack_res else null


func get_item_stack_data() -> ItemStackResource:
	return inventory_data.get_item(slot_index)


func set_item_stack_data(item_stack: ItemStackResource) -> void:
	inventory_data.insert_item(item_stack, slot_index)

