extends Control


const HOTBAR_KEY_INDEX_MAPPING: Dictionary = {
	'hotbar_1': 0,
	'hotbar_2': 1,
	'hotbar_3': 2,
	'hotbar_4': 3,
	'hotbar_5': 4,
	'hotbar_6': 5,
	'hotbar_7': 6,
	'hotbar_8': 7,
	'hotbar_9': 8,
	'hotbar_0': 9,
} 

const HOTBAR_OFFSET: Vector2 = Vector2(10, 10) 

@onready var hotbar_container: HotbarContainer = $HotbarContainer
@onready var build_container: BuildContainer = $BuildContainer
@onready var inventory_grid: InventoryGrid = $InventoryGrid
@onready var equipment_container: EquipmentContainer = $EquipmentContainer

@export var visible_inventory: bool = false


func _ready() -> void:
	show_inventory() if visible_inventory else hide_inventory()


func _unhandled_input(event: InputEvent) -> void:
	_handle_show_inventory_input(event)
	_handle_hotbar_select_input(event)


func show_inventory():
	inventory_grid.visible = true
	build_container.visible = true
	visible_inventory = true
	#equipment_container.visible = true


func hide_inventory():
	inventory_grid.visible = false
	build_container.visible = false
	Singletons.build_data_resource.current_selected_structure = -1
	visible_inventory = false
	#equipment_container.visible = false


func _handle_show_inventory_input(event: InputEvent) -> void:
	if event.is_action_pressed("inventory"):
		hide_inventory() if visible_inventory else show_inventory()
		get_viewport().set_input_as_handled()


func _handle_hotbar_select_input(event: InputEvent) -> void:
	for hotbar_key in HOTBAR_KEY_INDEX_MAPPING:
		if event.is_action_pressed(hotbar_key):
			hotbar_container.inventory_data.current_hotbar_index = HOTBAR_KEY_INDEX_MAPPING[hotbar_key]
			get_viewport().set_input_as_handled()


#func _handle_interact_item_input(event: InputEvent) -> void:
	#if event.is_action_pressed(INTERACT_ITEM_INPUT):
		#hotbar_container.inventory_data.interact_chosen_hotbar()
		#get_viewport().set_input_as_handled()
