class_name Plant
extends Building


const PICKUPABLE_SCN := preload("res://objects/pickupable/pickupable.tscn")
const PICKUPABLE_OFFSET := Vector2(20, 20)

@export var plant_data: PlantDataResource

var _current_growth_stage := -1

@onready var _sprite: Sprite2D = $Sprite2D
@onready var _animation_player: AnimationPlayer = $AnimationPlayer
@onready var _growth_timer: Timer = $GrowthTimer


func _ready() -> void:
	_growth_timer.start(plant_data.growth_time[_current_growth_stage])
	_grow()


func _on_input_event(viewport: Node, event: InputEvent, shape_idx: int) -> void:
	if event.is_action_pressed("interact_building"):
		_harvest()


func _grow() -> void:
	_current_growth_stage += 1
	_sprite.texture = plant_data.growth_sprites[_current_growth_stage]
	_animation_player.play("grow")


func _harvest() -> void:
	for i in plant_data.harvest_items_id.size():
		var item_res := ItemDatabase.get_item(plant_data.harvest_items_id[i])
		var item_quantity := roundi(plant_data.harvest_items_quantity[i] * plant_data.growth_harvest_factor[_current_growth_stage])
		
		printt(plant_data.harvest_items_id[i], item_quantity)
		
		if item_quantity <= 0:
			continue
		
		var item_stack_res := ItemStackResource.new()
		item_stack_res.init_stack(item_res, item_quantity)
		
		var pickupable: Pickupable = PICKUPABLE_SCN.instantiate()
		pickupable.item_stack_res = item_stack_res
		pickupable.global_position = global_position + PICKUPABLE_OFFSET + Vector2(randf_range(-20, 20), 0)
		
		# HACK: should get to static parent
		get_parent().add_child(pickupable)
	
	destroyed.emit()
	queue_free()


func _on_growth_timer_timeout() -> void:
	_grow()
	
	if _current_growth_stage < plant_data.max_growth:
		_growth_timer.start(plant_data.growth_time[_current_growth_stage])
