extends Node
class_name Ageing

signal age_changed(new_age:float, last_age:float)
signal age_treshold_reached(new_scene:Node2D)

@export var target : Node2D
@export var current_age = 0.0 :
	set(value):
		if (value != current_age):
			var last_age = current_age
			current_age = value
			emit_signal("age_changed", current_age, last_age)
			
			if(current_age >= age_treshold && !threshold_reached):
				var new_scene :Node2D
				
				if (next_scene != null):
					new_scene = _create_new_scene()
				emit_signal("age_treshold_reached",new_scene)
				threshold_reached = true
				target.queue_free()
@export var age_treshold = 1.0
@export_file("*.tscn") var next_scene

var threshold_reached = false
static var group_name = "Ageing"

func _ready():
	if (target == null):
		target = get_parent()
	add_to_group(group_name)
	
	var timer = Timer.new()
	timer.wait_time = 1.0
	timer.one_shot = false
	timer.connect("timeout", Callable(self, "_update_age"))
	add_child(timer)
	timer.start()

func _create_new_scene() -> Node2D:
	var instance : Node2D = load(next_scene).instantiate()
	target.get_parent().add_child(instance)
	instance.global_transform = target.global_transform
	return instance

func _update_age():
	for ageing in get_tree().get_nodes_in_group(Ageing.group_name):
		if(ageing is Ageing):
			ageing.current_age += 0.1
			print (ageing.current_age)
