class_name BuildController
extends Node2D


const MOUSE_OFFSET := Vector2(-20, -20)
const BUILDING_OFFSET := Vector2(0, 4)

@export var solid_tilemap: TileMap

var inventory_data := preload("res://scripts/inventory/player_inventory_data.tres")
var build_data := preload("res://scripts/build/build_data_res.tres")

## Vector2i -> Building
var structures := {}


#func _input(event):
	#if event.is_action_pressed("spacebar"):
		#print(inventory_data.count_items(["1_wood"]))


func _unhandled_input(event: InputEvent) -> void:
	# HACK: node order is important, input could be eaten by lower nodes
	if event.is_action_pressed("build"):
		if try_build_structure(build_data.get_current_structure()):
			get_viewport().set_input_as_handled()
		#if event.is_action_pressed("remove"):
			#get_viewport().set_input_as_handled()
			#_delete_structure()


func get_mouse_tile() -> Vector2i:
	var mouse_position := get_global_mouse_position() + MOUSE_OFFSET
	mouse_position.x = floor((mouse_position.x + (Grid.x_tile.x/2)) / Grid.x_tile.x)
	mouse_position.y = floor((mouse_position.y + (Grid.y_tile.y/2)) / Grid.y_tile.y)
	return Vector2i(mouse_position.x, mouse_position.y)


func is_mouse_tile_empty() -> bool:
	var tile_data := solid_tilemap.get_cell_tile_data(0, solid_tilemap.local_to_map(solid_tilemap.get_local_mouse_position()))
	return not is_instance_valid(structures.get(get_mouse_tile())) and tile_data == null


func is_mouse_tile_supported() -> bool:
	var below_tile_pos := get_mouse_tile() + Vector2i.DOWN
	var below_tile_data := solid_tilemap.get_cell_tile_data(0, solid_tilemap.local_to_map(solid_tilemap.get_local_mouse_position()) + Vector2i.DOWN)
	
	if below_tile_data != null:
		return true
	
	var building: Building = structures.get(below_tile_pos)
	if is_instance_valid(building) and not building is Plant:
		return true
	
	return false


## Used when there's already a Building that needs to be placed
func place_building_at_mouse(building: Building, need_support := false) -> bool:
	if not is_mouse_tile_empty():
		return false
	
	var grid_position := get_mouse_tile()
	
	if need_support and not is_mouse_tile_supported():
		return false
	
	add_child(building)
	var building_pos := Vector2(grid_position.x * Grid.x_tile.x, grid_position.y * Grid.y_tile.y) + BUILDING_OFFSET
	building.global_position = building_pos
	structures[grid_position] = building
	
	return true


## Used when player interface with the build UI
func try_build_structure(structure_res: StructureResource) -> bool:
	# cek selected
	if structure_res == null:
		return false
	
	if not is_mouse_tile_empty():
		return false
	
	# cek position
	var grid_position := get_mouse_tile()
	
	if structure_res.need_support and not is_mouse_tile_supported():
		return false
	
	# cek requirement
	var item_req_dict = {}
	for item_quantity: ItemQuantity in structure_res.item_req:
		item_req_dict[item_quantity.item_id] = item_quantity.quantity
	
	var item_count_dict = inventory_data.count_items(item_req_dict.keys())
	for key in item_req_dict.keys():
		if item_req_dict[key] > item_count_dict[key]:
			return false
	
	inventory_data.delete_items(item_req_dict)
	
	var building: Building = structure_res.structure_scn.instantiate()
	add_child(building)
	var building_pos := Vector2(grid_position.x * Grid.x_tile.x, grid_position.y * Grid.y_tile.y) + BUILDING_OFFSET
	building.global_position = building_pos
	structures[grid_position] = building
	
	return true


#func _delete_structure():
	#var mouse_position = get_global_mouse_position()
	#
	#mouse_position.x = floor((mouse_position.x + (Grid.x_tile.x/2)) / Grid.x_tile.x)
	#mouse_position.y = floor((mouse_position.y + (Grid.y_tile.y/2)) / Grid.y_tile.y)
	#var grid_position = Vector2i(mouse_position.x, mouse_position.y)
	#
	#if structures.has(grid_position):
		#var struct: Building = structures[grid_position]
		#structures.erase(grid_position)
		#struct.queue_free()
