extends Actor


@export var acceleration := 100.0
@export var speed := 300.0

# Get the gravity from the project settings to be synced with RigidBody nodes.
var gravity: int = ProjectSettings.get_setting("physics/2d/default_gravity")

@onready var _state_chart: StateChart = $StateChart
@onready var _hitbox: Hitbox = $Hitbox
@onready var _health_bar: ProgressBar = $HealthBar
@onready var _sprite: Sprite2D = $Sprite2D


func _ready() -> void:
	super._ready()
	
	_health_bar.max_value = stats.max_hp
	_health_bar.value = stats.hp
	stats.hp_changed.connect(
		func() -> void:
			_health_bar.value = stats.hp
	)
	
	_hitbox.actor_stats = stats


#region Idle State
func _on_idle_state_physics_processing(delta: float) -> void:
	velocity = velocity.move_toward(Vector2.ZERO, acceleration * delta)
	move_and_slide()
#endregion


#region Chase State
var tracked_player: Node2D

func _on_chase_state_physics_processing(delta: float) -> void:
	var direction := (tracked_player.global_position - global_position).normalized()
	_sprite.global_rotation = direction.angle()
	velocity = velocity.move_toward(direction * speed, acceleration * delta)
	move_and_slide()
#endregion


func _on_hitbox_hurtbox_detected(hurtbox: Hurtbox) -> void:
	_hitbox.try_hurt()


func _on_player_detector_player_detected(player: Node2D) -> void:
	tracked_player = player
	_state_chart.send_event("ToChase")


func _on_player_detector_player_undetected(player: Node2D) -> void:
	tracked_player = null
	_state_chart.send_event("ToIdle")


func _on_died() -> void:
	queue_free()
