extends Actor


const CHASE_DISTANCE := 150.0

@export var move_speed = 300.0
@export var jump_force = -400.0

# Get the gravity from the project settin gs to be synced with RigidBody nodes.
var gravity: int = ProjectSettings.get_setting("physics/2d/default_gravity")

@onready var _state_chart: StateChart = $StateChart
@onready var _rotating_pivot: Marker2D = $RotatingPivot
@onready var _sprite: Sprite2D = $Sprite2D
@onready var _left_wall_ray_cast: RayCast2D = $LeftWallRayCast
@onready var _right_wall_ray_cast: RayCast2D = $RightWallRayCast
@onready var _projectile_shooter: ProjectileShooter = $RotatingPivot/ProjectileShooter
@onready var _fire_cooldown_timer: Timer = $FireCooldownTimer
@onready var _health_bar: ProgressBar = $HealthBar


func _ready() -> void:
	super._ready()
	_projectile_shooter.actor_stats = stats
	
	_health_bar.max_value = stats.max_hp
	_health_bar.value = stats.hp
	stats.hp_changed.connect(
		func() -> void:
			_health_bar.value = stats.hp
	)


# ONLY TO BE CALLED IN _physics_process!
func _move(move_dir: float, jump: bool, delta: float) -> void:
	if not is_on_floor():
		velocity.y += gravity * delta
	
	if jump and is_on_floor():
		velocity.y = jump_force
	
	if not is_zero_approx(move_dir):
		if absf(move_dir) > 1:
			move_dir = signf(move_dir)
			_sprite.flip_h = move_dir < 0
		velocity.x = move_dir * move_speed
	else:
		velocity.x = move_toward(velocity.x, 0, move_speed)
	
	move_and_slide()


#region Idle state
func _on_idle_state_physics_processing(delta: float) -> void:
	_move(0, false, delta)
#endregion


#region Chase state
var tracked_player: Node2D

func _on_chase_state_entered() -> void:
	_fire_cooldown_timer.start()


func _on_chase_state_physics_processing(delta: float) -> void:
	_rotating_pivot.look_at(tracked_player.global_position)
	
	if absf(tracked_player.global_position.x - global_position.x) > CHASE_DISTANCE:
		var dir := tracked_player.global_position.x - global_position.x
		var jump := (signi(dir) < 0 and _left_wall_ray_cast.is_colliding()) or (signi(dir) > 0 and _right_wall_ray_cast.is_colliding())
		_move(dir, jump, delta)
	else:
		_move(0, false, delta)


func _on_chase_state_exited() -> void:
	_fire_cooldown_timer.stop()
#endregion


func _on_player_detector_player_detected(player: Node2D) -> void:
	tracked_player = player
	_state_chart.send_event("ToChase")


func _on_player_detector_player_undetected(player: Node2D) -> void:
	tracked_player = null
	_state_chart.send_event("ToIdle")


func _on_died() -> void:
	queue_free()


func _on_fire_cooldown_timer_timeout() -> void:
	_projectile_shooter.fire()
