extends Actor


const JUMP_INTERVAL := 2.0

@export var move_speed = 300.0
@export var jump_force = -400.0

# Get the gravity from the project settings to be synced with RigidBody nodes.
var gravity: int = ProjectSettings.get_setting("physics/2d/default_gravity")

var _jump_timer := JUMP_INTERVAL

@onready var _health_bar: ProgressBar = $HealthBar
@onready var _state_chart: StateChart = $StateChart
@onready var _hitbox: Hitbox = $Hitbox
@onready var _mouth_sprite: Sprite2D = $Sprite2D/MouthSprite


func _ready() -> void:
	super._ready()
	
	_health_bar.max_value = stats.max_hp
	_health_bar.value = stats.hp
	stats.hp_changed.connect(
		func() -> void:
			_health_bar.value = stats.hp
	)
	
	_hitbox.actor_stats = stats


func _physics_process(delta: float) -> void:
	_jump_timer -= delta


# ONLY TO BE CALLED IN _physics_process!
func _move(move_dir: float, delta: float) -> void:
	if absf(move_dir) > 1:
		move_dir = signf(move_dir)
	
	if not is_on_floor():
		velocity.y += gravity * delta
	else:
		velocity.x = move_toward(velocity.x, 0, move_speed)
		
		if not is_zero_approx(move_dir):
			if _jump_timer <= 0.0:
				_jump(move_dir)
				_jump_timer = JUMP_INTERVAL
	
	move_and_slide()


func _jump(move_dir: float) -> void:
	velocity.x = move_dir * move_speed
	velocity.y = jump_force


#region Idle state
func _on_idle_state_physics_processing(delta: float) -> void:
	_move(0, delta)
#endregion


#region Chase state
var tracked_player: Node2D

func _on_chase_state_physics_processing(delta: float) -> void:
	_mouth_sprite.position.x = lerpf(_mouth_sprite.position.x, signf(tracked_player.global_position.x - global_position.x) * 4, 0.2)
	_move(tracked_player.global_position.x - global_position.x, delta)
#endregion


func _on_hitbox_hurtbox_detected(hurtbox: Hurtbox) -> void:
	_hitbox.try_hurt()


func _on_died() -> void:
	queue_free()
