extends Actor

## This is a test enemy with walk movement type like zombie in terraria. Just to be used as a reference code for further planning and iteration.


@export var move_speed = 300.0
@export var jump_force = -400.0

@onready var state_chart: StateChart = $StateChart

# Get the gravity from the project settings to be synced with RigidBody nodes.
var gravity: int = ProjectSettings.get_setting("physics/2d/default_gravity")


# ONLY TO BE CALLED IN _physics_process!
func _move(move_dir: float, jump: bool, delta: float) -> void:
	if not is_on_floor():
		velocity.y += gravity * delta
	
	if jump and is_on_floor():
		velocity.y = jump_force
	
	if not is_zero_approx(move_dir):
		if absf(move_dir) > 1:
			move_dir = signf(move_dir)
		velocity.x = move_dir * move_speed
	else:
		velocity.x = move_toward(velocity.x, 0, move_speed)
	
	move_and_slide()


#region Idle state
func _on_idle_state_physics_processing(delta: float) -> void:
	_move(0, false, delta)
#endregion


#region Chase state
var tracked_player: Node2D

func _on_chase_state_physics_processing(delta: float) -> void:
	if absf(tracked_player.global_position.x - global_position.x) > 40.0:
		_move(tracked_player.global_position.x - global_position.x, tracked_player.global_position.y < global_position.y - 20, delta)
	else:
		_move(0, false, delta)
#endregion


func _on_player_detector_player_detected(player: Node2D) -> void:
	tracked_player = player
	state_chart.send_event("ToChase")


func _on_player_detector_player_undetected(player: Node2D) -> void:
	tracked_player = null
	state_chart.send_event("ToIdle")


func _on_died() -> void:
	queue_free()
