class_name TextWriter
extends RichTextLabel


signal write_finished()

## char / second
@export var _default_write_speed := 1.0

var _current_text := ""
var _write_speed := 1.0

var _is_writing := false
var _write_timer := 0.0
var _current_char_index := 0


func _process(delta: float) -> void:
		if not _is_writing:
				return

		if _current_char_index == _current_text.length():
				_is_writing = false
				write_finished.emit()
				return

		_write_timer -= delta
		if _write_timer <= 0.0:
				text += _current_text[_current_char_index]
				_write_timer += 1.0 / _write_speed
				_current_char_index += 1


func write(new_text: String, write_speed := _default_write_speed) -> void:
		_current_text = new_text
		_write_speed = write_speed

		_is_writing = true
		_write_timer = 1.0 / _write_speed
		_current_char_index = 0


func finish_text_immediately() -> void:
		text = _current_text
		_is_writing = false
		write_finished.emit()


func clear_text() -> void:
		text = ""
		_is_writing = false


func is_writing() -> bool:
		return _is_writing
