extends Building


const PICKUPABLE_SCN := preload("res://objects/pickupable/pickupable.tscn")
const PICKUPABLE_OFFSET := Vector2(20, 20)

@export var craft_panel: Panel
@export var slot_1: InventorySlot
@export var slot_2: InventorySlot

var craft_inventory := InventoryDataResource.new()


func _ready() -> void:
	slot_1.set_inventory_data(craft_inventory)
	slot_2.set_inventory_data(craft_inventory)
	
	craft_inventory.slot_changed.connect(
		func(slot_index: int) -> void:
			slot_1.update_item_stack_data()
			slot_2.update_item_stack_data()
	)
	craft_inventory.update_all.connect(
		func() -> void:
			slot_1.refresh_item_stack_data()
			slot_2.refresh_item_stack_data()
	)


func _input(event: InputEvent) -> void:
	# HACK: Most likely fine, hard to find cases where we don't want to close the panel.
	if event.is_action_pressed("interact_building"):
		craft_panel.hide()


func _craft() -> void:
	# TODO: drop potions according to the ingredients
	var items := craft_inventory.count_items(["1_apple"])
	var item_quantity: int = items.get("1_apple", 0)
	
	if item_quantity <= 0:
		return
	
	var item_res := ItemDatabase.get_item("1_elixir")
	
	var item_stack_res := ItemStackResource.new()
	item_stack_res.init_stack(item_res, item_quantity)
	
	var pickupable: Pickupable = PICKUPABLE_SCN.instantiate()
	pickupable.item_stack_res = item_stack_res
	pickupable.global_position = global_position + PICKUPABLE_OFFSET + Vector2(randf_range(-20, 20), 0)
	
	# HACK: parent is static
	get_parent().add_child(pickupable)
	
	craft_inventory.delete_items({"1_apple" : item_quantity})


func _on_input_event(viewport: Node, event: InputEvent, shape_idx: int) -> void:
	if event.is_action_pressed("interact_building"):
		craft_panel.show()


func _on_craft_button_pressed() -> void:
	_craft()
